require_relative "sanity/version"
require_relative "sanity/cli"
require 'typhoeus'

module Sanity
  def self.config
    @config ||= begin
                  if Rails.application.respond_to?(:config_for)
                    Rails.application.config_for(:sanity)
                  else
                    YAML.load_file(Rails.root.join('config','sanity.yml'))[Rails.env].symbolize_keys
                  end
                end
  end

  def self.token
    self.config[:token] || ENV['SANITY_TOKEN'] || $sanity_token
  end

  class Client
    def initialize(label=nil)
      @label = label
      @logs = []
      @success = true
    end

    def broken(obj)
      # obj can be string or hash
      # reports args back to HQ
      @logs << obj
      @success = false
    end

    def success(obj)
      # obj can be string or hash
      # reports args back to HQ
      @logs << obj
      @success = true
    end

    def report!
      Typhoeus::Request.new(
        Sanity.config[:url],
        method: :post,
        body: { data: @logs.to_json },
        params: { token: Sanity.token,
                  email: Sanity.config[:email],
                  label: @label,
                  success: @success }
      ).run
    end

  end
end

module Kernel
  def sanity(label=nil, &block)
    raise "ENV['SANITY_TOKEN'] || $sanity_token not set. Aborting." unless Sanity.token

    client = Sanity::Client.new(label)
    client.instance_eval(&block)
    client.report!
  end
end
